/*****
  command line parser -- generated by clig (Version: 1995-11-22)

  The command line parser `clig':
  (C) 1995 Harald Kirsch (kir@iitb.fhg.de)
*****/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

/* mainly a hack to make things work on SUNOS with gcc */
extern double strtod(const char *, char **);
extern long int strtol(const char *, char **, int);

#include <cmdline.h>

char *Program;


static Cmdline cmd = {
  /***** -s: do not print the settings of the driver */
  /* silentP = */ 0,
  /***** -c: channels to sample: 1,..,16, 1-4, 5-8, 9-12, 13-16, 1-8, 9-16, 1-16 */
  /* channelP = */ 0,
  /* channel = */ (char*)0,
  /* channelC = */ 0,
  /***** -r: restart driver and hardware on buffer/FIFO overflow */
  /* restartP = */ 0,
  /* restart = */ (int)0,
  /* restartC = */ 0,
  /***** -t: trigger mode: free, +edge, -edge, +level, -level, TTL */
  /* triggerP = */ 0,
  /* trigger = */ (char*)0,
  /* triggerC = */ 0,
  /***** -tl: relative trigger level, 0 for 0V */
  /* triggerLevelP = */ 0,
  /* triggerLevel = */ (int)0,
  /* triggerLevelC = */ 0,
  /***** -f: sampling frequency in Hz (don't use with -d */
  /* frequencyP = */ 0,
  /* frequency = */ (int)0,
  /* frequencyC = */ 0,
  /***** -d: divider for external clock (don't use with -f) */
  /* dividerP = */ 0,
  /* divider = */ (int)0,
  /* dividerC = */ 0,
  /***** -i: interrupt number to use (0 for polling mode) */
  /* irqP = */ 0,
  /* irq = */ (int)0,
  /* irqC = */ 0,
  /***** -pi: polling interval in jiffies (10ms) */
  /* pollintP = */ 0,
  /* pollint = */ (int)0,
  /* pollintC = */ 0,
  /***** -to: timeout in seconds for read operations (0: wait forever) */
  /* timeoutP = */ 0,
  /* timeout = */ (int)0,
  /* timeoutC = */ 0
};



/**********************************************************************/
int
getIntOpt(int argc, char **argv, int i, int *value, int force)
{
  char *end;

  if( ++i>=argc ) goto nothingFound;

  *value = (int)strtol(argv[i], &end, 10);

  /***** check for conversion error */
  if( end==argv[i] ) goto nothingFound;

  /***** check for surplus non-whitespace */
  while( isspace(*end) ) end+=1;
  if( *end ) goto nothingFound;

  return i;

nothingFound:
  if( !force ) return i-1;

  fprintf(stderr, 
	  "%s: missing or malformed integer value after option `%s'\n",
	  Program, argv[i-1]);
    exit(1);
}
/**********************************************************************/
int
getIntOpts(int argc, char **argv, int i, 
	   int **values,
	   int cmin, int cmax)
/*****
  We want to find at least cmin values and at most cmax values.
  cmax==-1 then means infinitely many are allowed.
*****/
{
  int alloced, used;
  char *end;

  if( i+cmin >= argc ) {
    fprintf(stderr, 
	    "%s: option `%s' wants at least %d parameters\n",
	    Program, argv[i], cmin);
    exit(1);
  }

  alloced = cmin + 4;
    
  *values = (int*)calloc(alloced, sizeof(int));
  if( ! *values ) {
outMem:
    fprintf(stderr, 
	    "%s: out of memory during parsing of option `%s'\n",
	    Program, argv[i]);
    exit(1);
  }

  for(used=0; (cmax==-1 || used<cmax) && used+i+1<argc; used++) {
    if( used==alloced ) {
      alloced += 8;
      *values = realloc(*values, alloced*sizeof(int));
      if( !*values ) goto outMem;
    }

    (*values)[used] = (int)strtol(argv[used+i+1], &end, 10);

    /***** check for conversion error */
    if( end==argv[i] ) break;

    /***** check for surplus non-whitespace */
    while( isspace(*end) ) end+=1;
    if( *end ) break;
  }
    
  if( used<cmin ) {
    fprintf(stderr, 
    "%s: less than %d parameters for option `%s', only %d found\n",
	    Program, cmin, argv[i], used);
    exit(1);
  }

  return i+used;
}
/**********************************************************************/
int
getFloatOpt(int argc, char **argv, int i, float *value, int force)
{
  char *end;

  if( ++i>=argc ) goto nothingFound;

  *value = (float)strtod(argv[i], &end);

  /***** check for conversion error */
  if( end==argv[i] ) goto nothingFound;

  /***** check for surplus non-whitespace */
  while( isspace(*end) ) end+=1;
  if( *end ) goto nothingFound;

  return i;

nothingFound:
  if( !force ) return i-1;

  fprintf(stderr,
	  "%s: missing or malformed float value after option `%s'\n",
	  Program, argv[i-1]);
  exit(1);
 
}
/**********************************************************************/
int
getFloatOpts(int argc, char **argv, int i, 
	   float **values,
	   int cmin, int cmax)
/*****
  We want to find at least cmin values and at most cmax values.
  cmax==-1 then means infinitely many are allowed.
*****/
{
  int alloced, used;
  char *end;

  if( i+cmin >= argc ) {
    fprintf(stderr, 
	    "%s: option `%s' wants at least %d parameters\n",
	    Program, argv[i], cmin);
    exit(1);
  }

  alloced = cmin + 4;
    
  *values = (float*)calloc(alloced, sizeof(float));
  if( ! *values ) {
outMem:
    fprintf(stderr, 
	    "%s: out of memory during parsing of option `%s'\n",
	    Program, argv[i]);
    exit(1);
  }

  for(used=0; (cmax==-1 || used<cmax) && used+i+1<argc; used++) {
    if( used==alloced ) {
      alloced += 8;
      *values = realloc(*values, alloced*sizeof(float));
      if( !*values ) goto outMem;
    }

    (*values)[used] = (float)strtod(argv[used+i+1], &end);

    /***** check for conversion error */
    if( end==argv[i] ) break;

    /***** check for surplus non-whitespace */
    while( isspace(*end) ) end+=1;
    if( *end ) break;
  }
    
  if( used<cmin ) {
    fprintf(stderr, 
    "%s: less than %d parameters for option `%s', only %d found\n",
	    Program, cmin, argv[i], used);
    exit(1);
  }

  return i+used;
}
/**********************************************************************/
int
getStringOpt(int argc, char **argv, int i, char **value, int force)
{
  if( ++i>=argc ) {
    fprintf(stderr, "%s: missing string after option `%s'\n",
            Program, argv[i-1]);
    exit(1);
  }
  
  if( !force && argv[i+1][0] == '-' ) return i-1;
  *value = argv[i];
  return i;
}
/**********************************************************************/
int
getStringOpts(int argc, char **argv, int i, 
	   char*  **values,
	   int cmin, int cmax)
/*****
  We want to find at least cmin values and at most cmax values.
  cmax==-1 then means infinitely many are allowed.
*****/
{
  int alloced, used;

  if( i+cmin >= argc ) {
    fprintf(stderr, 
	    "%s: option `%s' wants at least %d parameters\n",
	    Program, argv[i], cmin);
    exit(1);
  }

  alloced = cmin + 4;
    
  *values = (char**)calloc(alloced, sizeof(char*));
  if( ! *values ) {
outMem:
    fprintf(stderr, 
	    "%s: out of memory during parsing of option `%s'\n",
	    Program, argv[i]);
    exit(1);
  }

  for(used=0; (cmax==-1 || used<cmax) && used+i+1<argc; used++) {
    if( used==alloced ) {
      alloced += 8;
      *values = realloc(*values, alloced*sizeof(char*));
      if( !*values ) goto outMem;
    }

    if( argv[used+i+1][0]=='-' ) break;
    (*values)[used] = argv[used+i+1];
  }
    
  if( used<cmin ) {
    fprintf(stderr, 
    "%s: less than %d parameters for option `%s', only %d found\n",
	    Program, cmin, argv[i], used);
    exit(1);
  }

  return i+used;
}
/**********************************************************************/
void
checkIntLower(char *opt, int *values, int count, int max)
{
  int i;

  for(i=0; i<count; i++) {
    if( values[i]<=max ) continue;
    fprintf(stderr, 
	    "%s: parameter %d of option `%s' greater than max=%d\n",
	    Program, i+1, opt, max);
    exit(1);
  }
}
/**********************************************************************/
void
checkIntHigher(char *opt, int *values, int count, int min)
{
  int i;

  for(i=0; i<count; i++) {
    if( values[i]>=min ) continue;
    fprintf(stderr, 
	    "%s: parameter %d of option `%s' smaller than min=%d\n",
	    Program, i+1, opt, min);
    exit(1);
  }
}
/**********************************************************************/
void
checkFloatLower(char *opt, float *values, int count, float max)
{
  int i;

  for(i=0; i<count; i++) {
    if( values[i]<=max ) continue;
    fprintf(stderr, 
	    "%s: parameter %d of option `%s' greater than max=%f\n",
	    Program, i+1, opt, max);
    exit(1);
  }
}
/**********************************************************************/
void
checkFloatHigher(char *opt, float *values, int count, float min)
{
  int i;

  for(i=0; i<count; i++) {
    if( values[i]>=min ) continue;
    fprintf(stderr, 
	    "%s: parameter %d of option `%s' smaller than min=%f\n",
	    Program, i+1, opt, min);
    exit(1);
  }
}
/**********************************************************************/
void
showOptionValues(void)
{
  int i;

  /***** -s: do not print the settings of the driver */
  if( !cmd.silentP ) {
    printf("-s not found.\n");
  } else {
    printf("-s found:\n");
  }

  /***** -c: channels to sample: 1,..,16, 1-4, 5-8, 9-12, 13-16, 1-8, 9-16, 1-16 */
  if( !cmd.channelP ) {
    printf("-c not found.\n");
  } else {
    printf("-c found:\n");
    if( !cmd.channelC ) {
      printf("  no values\n");
    } else {
      printf("  value = `%s'\n", cmd.channel);
    }
  }

  /***** -r: restart driver and hardware on buffer/FIFO overflow */
  if( !cmd.restartP ) {
    printf("-r not found.\n");
  } else {
    printf("-r found:\n");
    if( !cmd.restartC ) {
      printf("  no values\n");
    } else {
      printf("  value = `%d'\n", cmd.restart);
    }
  }

  /***** -t: trigger mode: free, +edge, -edge, +level, -level, TTL */
  if( !cmd.triggerP ) {
    printf("-t not found.\n");
  } else {
    printf("-t found:\n");
    if( !cmd.triggerC ) {
      printf("  no values\n");
    } else {
      printf("  value = `%s'\n", cmd.trigger);
    }
  }

  /***** -tl: relative trigger level, 0 for 0V */
  if( !cmd.triggerLevelP ) {
    printf("-tl not found.\n");
  } else {
    printf("-tl found:\n");
    if( !cmd.triggerLevelC ) {
      printf("  no values\n");
    } else {
      printf("  value = `%d'\n", cmd.triggerLevel);
    }
  }

  /***** -f: sampling frequency in Hz (don't use with -d */
  if( !cmd.frequencyP ) {
    printf("-f not found.\n");
  } else {
    printf("-f found:\n");
    if( !cmd.frequencyC ) {
      printf("  no values\n");
    } else {
      printf("  value = `%d'\n", cmd.frequency);
    }
  }

  /***** -d: divider for external clock (don't use with -f) */
  if( !cmd.dividerP ) {
    printf("-d not found.\n");
  } else {
    printf("-d found:\n");
    if( !cmd.dividerC ) {
      printf("  no values\n");
    } else {
      printf("  value = `%d'\n", cmd.divider);
    }
  }

  /***** -i: interrupt number to use (0 for polling mode) */
  if( !cmd.irqP ) {
    printf("-i not found.\n");
  } else {
    printf("-i found:\n");
    if( !cmd.irqC ) {
      printf("  no values\n");
    } else {
      printf("  value = `%d'\n", cmd.irq);
    }
  }

  /***** -pi: polling interval in jiffies (10ms) */
  if( !cmd.pollintP ) {
    printf("-pi not found.\n");
  } else {
    printf("-pi found:\n");
    if( !cmd.pollintC ) {
      printf("  no values\n");
    } else {
      printf("  value = `%d'\n", cmd.pollint);
    }
  }

  /***** -to: timeout in seconds for read operations (0: wait forever) */
  if( !cmd.timeoutP ) {
    printf("-to not found.\n");
  } else {
    printf("-to found:\n");
    if( !cmd.timeoutC ) {
      printf("  no values\n");
    } else {
      printf("  value = `%d'\n", cmd.timeout);
    }
  }
  if( !cmd.argc ) {
    printf("no remaining parameters in argv\n");
  } else {
    printf("argv =");
    for(i=0; i<cmd.argc; i++) {
      printf(" `%s'", cmd.argv[i]);
    }
    printf("\n");
  }
}
/**********************************************************************/


void
usage(void)
{
  fprintf(stderr, "\
usage: %s [-s] [-c channel] [-r restart] [-t trigger] [-tl triggerLevel] [-f frequency] [-d divider] [-i irq] [-pi pollint] [-to timeout] device\n\
    Read and set parameters of rti860 device driver.\n\
      -s: do not print the settings of the driver\n\
      -c: channels to sample: 1,..,16, 1-4, 5-8, 9-12, 13-16, 1-8, 9-16, 1-16\n\
          1 string value\n\
      -r: restart driver and hardware on buffer/FIFO overflow\n\
          1 integer value between 0 and 1\n\
      -t: trigger mode: free, +edge, -edge, +level, -level, TTL\n\
          1 string value\n\
     -tl: relative trigger level, 0 for 0V\n\
          1 integer value between -127 and 127\n\
      -f: sampling frequency in Hz (don't use with -d\n\
          1 integer value between 1 and 200000\n\
      -d: divider for external clock (don't use with -f)\n\
          1 integer value between 1 and oo\n\
      -i: interrupt number to use (0 for polling mode)\n\
          1 integer value between 0 and 15\n\
     -pi: polling interval in jiffies (10ms)\n\
          1 integer value between 1 and oo\n\
     -to: timeout in seconds for read operations (0: wait forever)\n\
          1 integer value between 0 and oo\n\
  device: device special file to examine\n\
version: |VERSION|\n\
", Program);
  exit(1);
}
/**********************************************************************/

Cmdline *
parseCmdline(int argc, char **argv)
{
  int i, keep;

  Program = argv[0];
  for(i=1, cmd.argc=1; i<argc; i++) {
    if( 0==strcmp("-s", argv[i]) ) {
      cmd.silentP = 1;
      continue;
    }

    if( 0==strcmp("-c", argv[i]) ) {
      cmd.channelP = 1;
      keep = i;
      i = getStringOpt(argc, argv, i, &cmd.channel, 1);
      cmd.channelC = i-keep;
      continue;
    }

    if( 0==strcmp("-r", argv[i]) ) {
      cmd.restartP = 1;
      keep = i;
      i = getIntOpt(argc, argv, i, &cmd.restart, 1);
      cmd.restartC = i-keep;
      checkIntLower("-r", &cmd.restart, cmd.restartC, 1);
      checkIntHigher("-r", &cmd.restart, cmd.restartC, 0);
      continue;
    }

    if( 0==strcmp("-t", argv[i]) ) {
      cmd.triggerP = 1;
      keep = i;
      i = getStringOpt(argc, argv, i, &cmd.trigger, 1);
      cmd.triggerC = i-keep;
      continue;
    }

    if( 0==strcmp("-tl", argv[i]) ) {
      cmd.triggerLevelP = 1;
      keep = i;
      i = getIntOpt(argc, argv, i, &cmd.triggerLevel, 1);
      cmd.triggerLevelC = i-keep;
      checkIntLower("-tl", &cmd.triggerLevel, cmd.triggerLevelC, 127);
      checkIntHigher("-tl", &cmd.triggerLevel, cmd.triggerLevelC, -127);
      continue;
    }

    if( 0==strcmp("-f", argv[i]) ) {
      cmd.frequencyP = 1;
      keep = i;
      i = getIntOpt(argc, argv, i, &cmd.frequency, 1);
      cmd.frequencyC = i-keep;
      checkIntLower("-f", &cmd.frequency, cmd.frequencyC, 200000);
      checkIntHigher("-f", &cmd.frequency, cmd.frequencyC, 1);
      continue;
    }

    if( 0==strcmp("-d", argv[i]) ) {
      cmd.dividerP = 1;
      keep = i;
      i = getIntOpt(argc, argv, i, &cmd.divider, 1);
      cmd.dividerC = i-keep;
      checkIntHigher("-d", &cmd.divider, cmd.dividerC, 1);
      continue;
    }

    if( 0==strcmp("-i", argv[i]) ) {
      cmd.irqP = 1;
      keep = i;
      i = getIntOpt(argc, argv, i, &cmd.irq, 1);
      cmd.irqC = i-keep;
      checkIntLower("-i", &cmd.irq, cmd.irqC, 15);
      checkIntHigher("-i", &cmd.irq, cmd.irqC, 0);
      continue;
    }

    if( 0==strcmp("-pi", argv[i]) ) {
      cmd.pollintP = 1;
      keep = i;
      i = getIntOpt(argc, argv, i, &cmd.pollint, 1);
      cmd.pollintC = i-keep;
      checkIntHigher("-pi", &cmd.pollint, cmd.pollintC, 1);
      continue;
    }

    if( 0==strcmp("-to", argv[i]) ) {
      cmd.timeoutP = 1;
      keep = i;
      i = getIntOpt(argc, argv, i, &cmd.timeout, 1);
      cmd.timeoutC = i-keep;
      checkIntHigher("-to", &cmd.timeout, cmd.timeoutC, 0);
      continue;
    }

    if( argv[i][0]=='-' ) {
      fprintf(stderr, "\n%s: unknown option `%s'\n\n",
              Program, argv[i]);
      usage();
    }
    argv[cmd.argc++] = argv[i];
  }/* for i */


  cmd.argv = argv+1;
  cmd.argc -= 1;

  if( 1>cmd.argc ) {
    fprintf(stderr, "%s: there should be at least 1 non-option argument(s)\n",
            Program);
    exit(1);
  }
  if( 1<cmd.argc ) {
    fprintf(stderr, "%s: there should be at most 1 non-option argument(s)\n",
            Program);
    exit(1);
  }
  return &cmd;
}

