#!/bin/csh -f


if ( $#argv != 1 ) then 
  echo "usage: $0 destDir"
  echo "    destDir: the directory where to put the .tar.gz-file"
  exit 1
endif

##### set destination dir
set dstDir = $argv[1]
if ( $dstDir == . ) set dstDir = $cwd

##### create version string
set ver = `date +%Y-%m-%d`

##### deduce the name of this package from cwd
set name = $cwd:t

rm -f $name.$ver.tar.gz

##### the tar is made from the parent directory
cd ..
if ( ! -d $name ) then
  echo "${0}: something is wrong, perhaps a misleading soft link."
  echo "    After changing dir to '..', there is no dir '$name'."
  exit 1
endif

##### let the user write release notes
if ( ! -e $name/RELNOTES.$ver ) then
  echo " " >$name/RELNOTES.$ver
  echo "----- end of release notes for $name.$ver -----"\
	>>$name/RELNOTES.$ver
endif
vi $name/RELNOTES.$ver


##### lets have a clean .depend-file
#mv $name/.depend $name/depend.orig
#touch $name/.depend

##### enter the version string into the makefile
cp $name/Makefile $name/Makefile.orig
sed '/^VERSION =/s/[0-9][0-9-]*/'$ver/ $name/Makefile.orig \
	>$name/Makefile
rm -f $name/Makefile.orig

##### generate exclude file
rm -f .exclude
echo $name/.exclude >$name/.exclude
echo $name/$name.$ver.tar.gz >>$name/.exclude

find $name \( \
	-name '*.tar.gz'  \
	-o -name '*~'     \
	-o -name '#*'     \
	-o -name core     \
	-o -name '*.[ao]' \
        -o -name '*.orig' \
	-o -name '*.cache' \
	-o -name '*.log' \
	-o -name '*.status' \
	-o -name config.h \
	-o \( -perm -100 -a \! -type d -a -size +100k \) \
        \) -print \
	>>$name/.exclude

tar cfXv - $name/.exclude $name |gzip >$dstDir/$name.$ver.tar.gz
echo "${0}: excluded files are listed in .exclude"

##### clean up
#mv $name/depend.orig $name/.depend

##### end #####
