/*****************************************************************
  Linux device driver for RTI860 hardware
  
  Copyright (C) 1995 Ralf Haller
            (C) 1995, 1996 Harald Kirsch (kir@iitb.fhg.de)

	    Fraunhofer Institut fr 
	    Informations- und Datenverarbeitung (IITB)
	    Fraunhoferstr. 1
	    76131 Karlsruhe

*****************************************************************/
/*
 *  values for ioctl
 */
#define RTI860_SET_PAR	1
#define RTI860_GET_PAR	2
#define RTI860_START	3
#define RTI860_RESET	4

/*
 * flags
 */
#define RTI860_F_PRESENT	(1<<0)
#define RTI860_F_OPEN		(1<<1)
#define RTI860_F_BUSY		(1<<2)
#define RTI860_F_IRQ		(1<<3)
#define RTI860_F_RESTART	(1<<4)

/* The following flag(s) can be changed by the user */
#define RTI860_F_USER		(RTI860_F_RESTART)

typedef enum {
  Channel16    =-16,
  Channel15    =-15,
  Channel14    =-14,
  Channel13    =-13,
  Channel12    =-12,
  Channel11    =-11,
  Channel10    =-10,
  Channel9     =-9,
  Channel8     =-8,
  Channel7     =-7,
  Channel6     =-6,
  Channel5     =-5,
  Channel4     =-4,
  Channel3     =-3,
  Channel2     =-2,
  Channel1     =-1,
  Channel1_4   = 0,
  Channel5_8   = 1,
  Channel9_12  = 2,
  Channel13_16 = 3,
  Channel1_16  = 4,
  Channel1_8   = 5,
  Channel9_16  = 6
} Channel;

typedef enum {
  Free         = 0,
  BereichPlus  = 1,
  BereichMinus = 2,
  PegelPlus    = 3,
  PegelMinus   = 4,
  TTLTrigger   = 5
} Trigger;

/*
 * parameter struct used for ioctl
 */
struct rti860_t
{
  unsigned int flags;		/* flags */
  unsigned int base;		/* base address */
  short channel;		/* channels to scan, see enum above */
  unsigned short trigger;	/* trigger mode, see enum above */
  signed char level;		/* trigger level, a value between -128
				   and 127 */
  int tics;			/* >0: use the 5MHz internal clock with
				   ticks as the divider.
				   <0: use the external clock with ticks
				   as the divider.
				   In both cases the sampling frequency
				   is 1.0/(ticks*base_frequency) */
  int irq;			/* interrupt channel, 0 if none */
  unsigned timeout;		/* timeout in seconds */
  unsigned sleep_time;		/* sleep time in polling mode in jiffies
				 */
};

typedef struct rti860_t Rti860;

#define F2TICS(f)	(5e6/(f))
#define TICS2F(tics)	(5e6/(tics))
